#include "Framework.h"
#include <iostream>

using namespace std;
using namespace Gestalt;

class PrintMod : public Module
{
public:
    static Module* make();

private:
    PrintMod() : Module("printmod") {};

    void _init()
    {
        connect("print", makeSlot(this, &PrintMod::print));
    }

    void print(Event* event)
    {
        ActorHnd target = event->getTarget();
        if (target != 0)
        {
            cout << _actors[target]->get<string>("message") << endl;
        }
        else // no target, print all
        {
            ActorMap::iterator pos = _actors.begin();
            for ( ; pos != _actors.end(); ++pos)
            {
                cout << pos->second->get<string>("message") << endl;
            }
        }
    }
};

Module* PrintMod::make()
{
    return new PrintMod;
};

class AState : public State
{
private:
    int count;
    ActorHnd foo;
    ActorHnd bar;

    void _init()
    {
        count = 0;
        PatternHnd fooPatt = addPattern("foo.pattern.xml");
        foo = addActor(fooPatt);

        addPattern("bar.pattern.xml");
        bar = addActor("bar");
        
        connect("AStop", makeSlot(this, &AState::AStop));
    }

    void _update(double fps)
    {
        ++count;
        
        if (count < 2)
            sendEvent(new Event("print"));
        else
            sendEvent(new Event("print", foo));
        
        if (count > 4)
            sendEvent(new Event("AStop"));
    }
    
    void AStop()
    {
        stop();
    }
};

int main (int argc, char* argv[])
{
    Framework fw;
    AState a;
    
    fw.addModule("printmod", &PrintMod::make);
    
    fw.push(&a);
    a.start();

    fw.start();
}
